#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Nov 13 11:19:14 2021

@author: kaustav
"""

import modelx as mx

mx.new_model("EV_Model")

Life = mx.new_space("Life")

# References for Life space
Life.death_rate = 0.03
Life.lapse_rate = 0.05
Life.pol_term = 20

# Fucntions in Life space
@mx.defcells(Life)
def prob_death(t):
    if t <= 0 or t > pol_term:
        return 0
    else:
        return prob_if_e(t-1) * death_rate

@mx.defcells(Life)
def prob_lapse(t):
    if t <= 0 or t > pol_term:
        return 0
    else:
        return prob_if_e(t-1) * (1 - death_rate) * lapse_rate

@mx.defcells(Life)
def prob_if_e(t):
    if t == 0:
        return 1
    else:
        return prob_if_e(t-1) - prob_death(t) - prob_lapse(t)

Term_Life = mx.new_space("Term_Life", bases=Life)

# References in Term_Life space
Term_Life.death_sa = 1000
Term_Life.ann_prem = 20
Term_Life.int_rate = 0.05

# Functions in Term_Life space
@mx.defcells(Term_Life)
def benefits(t):
    if t <= 0 or t > pol_term:
        return 0
    else:
        return death_sa * prob_death(t)
    
@mx.defcells(Term_Life)
def premiums(t):
    if t <= 0 or t > pol_term:
        return 0
    else:
        return ann_prem * prob_if_e(t-1)
    
@mx.defcells(Term_Life)
def pv_benefits(t):
    if t <= 0 or t > pol_term:
        return 0
    else:
        return (benefits(t+1) + pv_benefits(t+1)) / (1 + int_rate)

@mx.defcells(Term_Life)
def pv_premiums(t):
    if t <= 0 or t > pol_term:
        return 0
    else:
        return premiums(t+1) + pv_premiums(t+1) / (1 + int_rate)
    
Term_Life.pv_benefits(1)
Term_Life.pv_premiums(1)
Term_Life.frame

# Creating a reserving sapce for the Term Life product
Term_Life_Res = mx.new_space("Term_Life_Res", bases=Term_Life)

# Setting more prudent reserving assumptions
Term_Life_Res.death_rate = Term_Life.death_rate * 1.1
Term_Life_Res.lapse_rate = Term_Life.lapse_rate * 0.9
Term_Life_Res.int_rate = Term_Life.int_rate * 0.9

@mx.defcells(Term_Life_Res)
def reseve_pp(t):
    return (pv_benefits(t) - pv_premiums(t)) / prob_if_e(t)

Term_Life_Res.reseve_pp(1)

# Creating a space to calculate PVFP for Term Life
Term_Life_PVFP = mx.new_space("Term_Life_PVFP", bases=Term_Life_Res)

# Save the model
mx.write_model(mx.EV_Model, "EV_Model", backup=False)